/*********************************************************************NVMH3****
File:  $Id: glossVertex1_tex.fx,v 1.1 2004/03/22 17:48:01 holger Exp $

Copyright NVIDIA Corporation 2002
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
    A glossy textured surface. The highlight is done in VERTEX shading --
	not as a texture. A version with pixel hilighting, requiring a
	special gloss texture, is forthcoming.
    Glossiness is controlled not only by the usual power function, but also
        by applying a set of gloss controls that cause a sharp falloff across
	a specified range.
    The falloff will occur in the highlight range [glossBot-glossTop] and the
        amount of falloff is specified by "glossDrop." Setting "glossDrop"
	to 1.0 nullifies the effect.

******************************************************************************/

/************* TWEAKABLES **************/

float4x4 WorldIT : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WorldViewProj : WorldViewProjection < string UIWidget="None"; >;
float4x4 World : World < string UIWidget="None"; >;
float4x4 ViewI : ViewInverse < string UIWidget="None"; >;

//////

float4 LightPos : Position
<
    string Object = "PointLight";
    string Space = "World";
> = {100.0f, 100.0f, 100.0f, 0.0f};

float4 AmbiColor : Ambient
<
	string UIWidget = "Color";
> = {0.1f, 0.1f, 0.1f, 1.0f};

float4 DiffColor : DIFFUSE
<
	string UIWidget = "Color";
> = {0.9f, 1.0f, 0.9f, 1.0f};

float4 SpecColor : Specular
<
	string UIWidget = "Color";
> = {0.7f, 0.7f, 1.0f, 1.0f};

float SpecExpon : SpecularPower
<
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName =  "specular power";
> = 12.0;

float GlossTop
<
    string UIWidget = "slider";
    float UIMin = 0.2;
    float UIMax = 1.0;
    float UIStep = 0.05;
    string UIName = "Bright Glossy Edge";
> = 0.7;

float GlossBot
<
    string UIWidget = "slider";
    float UIMin = 0.05;
    float UIMax = 0.95;
    float UIStep = 0.05;
    string UIName = "Dim Glossy Edge";
> = 0.5;

float GlossDrop
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.0;
    float UIStep = 0.05;
    string UIName = "Glossy Brightness Drop";
> = 0.2;

/////////////

texture colorTexture : DIFFUSE
<
    string ResourceName = "default_color.dds";
    string TextureType = "2D";
>;

sampler2D colorTextureSampler = sampler_state
{
	Texture = <colorTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float3 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float4 TexCoord0	: TEXCOORD0;
    float4 diffCol	: COLOR0;
    float4 specCol	: COLOR1;
};

/*********** vertex shader ******/

vertexOutput glossVS(appdata IN) {
    vertexOutput OUT;
    float3 Nn = normalize(mul(IN.Normal, WorldIT).xyz);
    float4 Po = float4(IN.Position.xyz,1);
    //compute worldspace position
    float3 Pw = mul(Po, World).xyz;
    float3 Ln = normalize(LightPos - Pw);
    float ldn = dot(Ln,Nn);
    float diffComp = max(0,ldn);
    float4 diffContrib = diffComp * DiffColor;
    diffContrib.w = 1.0;
    OUT.diffCol = diffContrib + AmbiColor;
    OUT.TexCoord0 = float4(IN.UV, 1);
    float3 EyePos = ViewI[3].xyz;
    float3 Vn = normalize(EyePos - Pw);
    float3 Hn = normalize(Vn + Ln);
    float hdn = pow(max(0,dot(Hn,Nn)),SpecExpon);
    hdn = hdn * (GlossDrop+smoothstep(GlossBot,GlossTop,hdn)*(1.0-GlossDrop));
    OUT.specCol = hdn * SpecColor;
    // transform into homogeneous-clip space
    OUT.HPosition = mul(Po, WorldViewProj);
    return OUT;
}

/********* pixel shader ********/

float4 glossPS_t(vertexOutput IN) : COLOR {
    return (IN.diffCol * tex2D(colorTextureSampler, IN.TexCoord0.xy) + IN.specCol);
}

/*************/

technique textured
{
    pass p0 
    {		
		VertexShader = compile vs_1_1 glossVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_1_1 glossPS_t();
    }
}

/***************************** eof ***/
