/*********************************************************************NVMH3****
File:  $Id: scene_DropShadow.fx,v 1.1 2004/03/22 17:48:01 holger Exp $

Copyright NVIDIA Corporation 2002
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
    This example shows a plastic surface, but the interesting part is the
        post-process.
    EffectEdit compatible, but EE won't know about RTT & won't load all the transforms.
    Blur routines are included via "blur59.fxh"

<scene_commands>
<clear color="0.1,0.1,0.1,0" depth="1.0"/>
<setcolortarget texture="GlowMap1"/>
<setdepthstenciltarget texture="DepthBuffer"/>
	<clear color="0.1,0.1,0.1,0" depth="1.0"/>
	<settechnique name="dropshad9"/>
	<setpass name="white"/>
	<draw type="objects_override"/>
<setcolortarget texture="GlowMap2"/>
	<setpass name="BlurGlowBuffer_Horz"/>
	<draw type="buffer"/>
<setcolortarget texture="GlowMap1"/>
	<setpass name="BlurGlowBuffer_Vert"/>
	<draw type="buffer"/>
<setcolortarget/>
<setdepthstenciltarget/>
	<setpass name="dropShad"/>
	<draw type="buffer"/>
	<draw type="objects"/>
</scene_commands>

******************************************************************************/

#include "blur59.fxh"

/************* UN-TWEAKABLES **************/

QUAD_REAL4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; > = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
QUAD_REAL4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
QUAD_REAL4x4 WorldXf : World < string UIWidget="None"; >;
QUAD_REAL4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

/*********** Tweakables **********************/

QUAD_REAL3 LightDirD : Direction
<
    string UIName = "Light Direction"; 
    string Object = "DirectionalLight";
    string Space = "World";
> = {-10.0f, 15.0f, 30.0f};

QUAD_REAL3 LightColorD
<
    string UIName =  "Distant Light Color";
    string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f};

QUAD_REAL LightIntensityD
<
    string UIName =  "Distant Light Strength";
> = 1.5;

///// ambient light

QUAD_REAL3 AmbiColor : Ambient
<
    string UIName =  "Ambient Color";
    string UIWidget = "Color";
> = {0.1f, 0.1f, 0.1f};

///// Surface Parameters ///////////////////////

QUAD_REAL4 SurfColor : DIFFUSE
<
    string UIName =  "Surface Color";
    string UIWidget = "Color";
> = {0.8f, 0.8f, 1.0f, 1.0f};

QUAD_REAL Ks
<
    string UIWidget = "slider";
    QUAD_REAL UIMin = 0.0;
    QUAD_REAL UIMax = 1.0;
    QUAD_REAL UIStep = 0.01;
    string UIName = "specular intensity";
> = 0.5;

QUAD_REAL SpecExpon : SpecularPower
<
    string UIWidget = "slider";
    QUAD_REAL UIMin = 1.0;
    QUAD_REAL UIMax = 128.0;
    QUAD_REAL UIStep = 1.0;
    string UIName =  "specular power";
> = 30.0;

// reflection parameters
QUAD_REAL Kr
<
    string UIWidget = "slider";
    QUAD_REAL UIMin = 0.0;
    QUAD_REAL UIMax = 1.0;
    QUAD_REAL UIStep = 0.01;
    string UIName =  "max reflection strength, along edges";
> = 1.0;

QUAD_REAL KrMin
<
    string UIWidget = "slider";
    QUAD_REAL UIMin = 0.0;
    QUAD_REAL UIMax = 1.0;
    QUAD_REAL UIStep = 0.01;
    string UIName =  "min reflection strength, facing you";
> = 0.05;

// reducing this gives a more "thick clearcoat" appearance
QUAD_REAL FresnelExpon : SpecularPower
<
    string UIWidget = "slider";
    QUAD_REAL UIMin = 1.0;
    QUAD_REAL UIMax = 5.0;
    QUAD_REAL UIStep = 0.1;
    string UIName =  "Schlick Approximation Exponent";
> = 5.0;

///// for RTT

// RTT Texture Declarations

DECLARE_QUAD_TEX(GlowMap1,GlowSamp1,"A8R8G8B8")
DECLARE_QUAD_TEX(GlowMap2,GlowSamp2,"A8R8G8B8")
DECLARE_QUAD_DEPTH_BUFFER(DepthBuffer, "D24S8")

// dropshadow-related tweakables

QUAD_REAL2 Shift = {-0.07,-0.05};

QUAD_REAL4 BgColor <
    string UIName =  "Surface Color";
    string UIWidget = "Color";
> = {0.8f, 0.5f, 0.4f, 1.0f};

QUAD_REAL4 ShadColor <
    string UIName =  "Surface Color";
    string UIWidget = "Color";
> = {0.2f, 0.2f, 0.1f, 1.0f};

/// user-defined textures ///////////////////////////////////

texture ColorTexture : DIFFUSE
<
    string ResourceName = "default_color.dds";
    string TextureType = "2D";
>;

sampler2D ColorSamp = sampler_state
{
	Texture = <ColorTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

/// environment map ///

texture EnvTexture : ENVIRONMENT
<
    string ResourceName = "default_reflection.dds";
    string TextureType = "Cube";
>;

samplerCUBE EnvSampler = sampler_state
{
	Texture = <EnvTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
    QUAD_REAL3 Position	: POSITION;
    QUAD_REAL4 UV		: TEXCOORD0;
    QUAD_REAL4 Normal	: NORMAL;
};

// vertex outputs - from vert shader to pixel shader //

// used for ambient/refl pass
struct ambiReflVOut {
    QUAD_REAL4 HPosition	: POSITION;
    QUAD_REAL2 UV		: TEXCOORD0;
    QUAD_REAL3 WorldNormal	: TEXCOORD1;
    QUAD_REAL3 WorldEyeVec	: TEXCOORD2;
};

// used for all other passes
struct vertexOutput {
    QUAD_REAL4 HPosition	: POSITION;
    QUAD_REAL2 UV		: TEXCOORD0;
    QUAD_REAL3 LightVec	: TEXCOORD1;
    QUAD_REAL3 WorldNormal	: TEXCOORD2;
    QUAD_REAL3 WorldEyeVec	: TEXCOORD3;
};

/*********** vertex shaders ******/

// all Vertex shaders in this FX file will need to determine
//   clipspace position, worldspace normals, an eye vector & texture coords.
//   "Pw" is used to determine point-lighting vectors
void sharedVS (appdata IN,
    out QUAD_REAL4 Ph,
    out QUAD_REAL3 N,
    out QUAD_REAL3 V,
    out QUAD_REAL2 UV,
    out QUAD_REAL3 Pw)
{
    N = mul(IN.Normal, WorldITXf).xyz;
    QUAD_REAL4 Po = QUAD_REAL4(IN.Position.xyz,1);
    Pw = mul(Po, WorldXf).xyz;
    V = (ViewIXf[3].xyz - Pw);
    Ph = mul(Po, WvpXf);
    UV = IN.UV.xy;
}

// vertex shader for ambient/reflection pass
ambiReflVOut ambiReflVS(appdata IN) {
    ambiReflVOut OUT; QUAD_REAL3 Pw;
    sharedVS(IN,OUT.HPosition,OUT.WorldNormal,OUT.WorldEyeVec,OUT.UV,Pw);
    return OUT;
}

// vertex shader for point lights
vertexOutput pointlightVS(appdata IN,
    uniform QUAD_REAL3 LightPos
) {
    vertexOutput OUT; QUAD_REAL3 Pw;
    sharedVS(IN,OUT.HPosition,OUT.WorldNormal,OUT.WorldEyeVec,OUT.UV,Pw);
    OUT.LightVec = (LightPos - Pw);
    return OUT;
}

// vertex shader for distant (directional) lights
vertexOutput distantlightVS(appdata IN,
    uniform QUAD_REAL3 LightDir
) {
    vertexOutput OUT; QUAD_REAL3 Pw;
    sharedVS(IN,OUT.HPosition,OUT.WorldNormal,OUT.WorldEyeVec,OUT.UV,Pw);
    OUT.LightVec = -LightDir;
    return OUT;
}

//////////////////////////////////
/********* pixel shaders ********/
//////////////////////////////////

QUAD_REAL4 whitePS(ambiReflVOut IN) : COLOR {
    return QUAD_REAL4(1,1,1,1);
}

// ambient/refl pass
QUAD_REAL4 ambiReflPS(ambiReflVOut IN) : COLOR {
    QUAD_REAL4 diffContrib, specContrib;
    QUAD_REAL3 Vn = normalize(IN.WorldEyeVec);
    QUAD_REAL3 Nn = normalize(IN.WorldNormal);
    QUAD_REAL4 ambiContrib = QUAD_REAL4(AmbiColor,1)*SurfColor*tex2D(ColorSamp,IN.UV);
    QUAD_REAL vdn=1-abs(dot(Vn,Nn));
    QUAD_REAL fres = KrMin + (Kr-KrMin) * pow(vdn,FresnelExpon);
    QUAD_REAL3 rv = reflect(Vn,Nn);
    QUAD_REAL4 refl = fres * texCUBE(EnvSampler,rv);
    return ambiContrib + refl;
}

// this portion shared for all lamps
QUAD_REAL4 sharedPS(vertexOutput IN,
    QUAD_REAL4 DiffColor,
    QUAD_REAL3 LightColor,
    QUAD_REAL Intensity
) {
    QUAD_REAL3 Ln = normalize(IN.LightVec);
    QUAD_REAL3 Vn = normalize(IN.WorldEyeVec);
    QUAD_REAL3 Nn = normalize(IN.WorldNormal);
    QUAD_REAL3 Hn = normalize(Vn + Ln);
    QUAD_REAL4 lv = lit(dot(Ln,Nn),dot(Hn,Nn),SpecExpon);
    QUAD_REAL4 diffContrib = DiffColor * QUAD_REAL4((Intensity*lv.y*LightColor + AmbiColor),1);
    QUAD_REAL4 specContrib = Ks * lv.z * QUAD_REAL4(Intensity*LightColor,0);
    return diffContrib + specContrib;
}

// lamps without falloff
QUAD_REAL4 lampPS(vertexOutput IN,
    uniform QUAD_REAL3 LightColor,
    uniform QUAD_REAL Intensity
) : COLOR {
    return sharedPS(IN,(SurfColor*tex2D(ColorSamp,IN.UV)),LightColor,Intensity);
}

// quadratic lamp
QUAD_REAL4 quadraticPS(vertexOutput IN,
    uniform QUAD_REAL3 LightColor,
    uniform QUAD_REAL Intensity
) : COLOR {
    QUAD_REAL atten = length(IN.LightVec);
    return sharedPS(IN,(SurfColor*tex2D(ColorSamp,IN.UV)),LightColor,Intensity/(atten*atten));
}

// declaration of "ScreenAlignedQuadVOut" in is "blur59.fxh" -- simple passthrough

QUAD_REAL4 dropPS(QuadVertexOutput IN) : COLOR
{   
    QUAD_REAL4 tex = tex2D(GlowSamp1, QUAD_REAL2(IN.UV+Shift));
    return lerp(BgColor,ShadColor,tex.x);
}  

/*************************/
/*** TECHNIQUES **********/
/*************************/

technique dropshad9
{
    pass white 
    {		
		VertexShader = compile vs_2_0 ambiReflVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 whitePS();
    }
    // At this point our 3D outline is rendered.
    // Time to blur.
    BLUR9_PASSES(GlowSamp1,"GlowMap2",GlowSamp2,"GlowMap1")
    pass dropShad
    {
		VertexShader = compile vs_1_1 ScreenQuadVS();
		cullmode = none;
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = false;
		// SrcBlend = one;
		// DestBlend = one;
		PixelShader = compile ps_2_0 dropPS();	
    }
    // now lay down ambient, reflection, and Z
    //    subsequent passes need not save Z, and they will
    //	be faster because of implicit Z occlusion
    pass ambiRefl 
    {		
		VertexShader = compile vs_2_0 ambiReflVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 ambiReflPS();
    }
    // directional light contribution
    pass distantlight1 
    {		
		VertexShader = compile vs_2_0 distantlightVS(LightDirD);
		ZEnable = true;
		ZWriteEnable = false;
		ZFunc = LessEqual;
		CullMode = None;
		AlphaBlendEnable = true;
		SrcBlend = One;
		DestBlend = One;
		PixelShader = compile ps_2_0 lampPS(LightColorD,LightIntensityD);
    }
}

//////// smaller dropshadow

technique dropshad5
{
    pass white
    {		
		VertexShader = compile vs_2_0 ambiReflVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 whitePS();
    }
    // At this point our 3D outline is rendered.
    // Time to blur.
    BLUR5_PASSES(GlowSamp1,"GlowMap2",GlowSamp2,"GlowMap1")
    pass dropShad
    {
		VertexShader = compile vs_1_1 ScreenQuadVS();
		cullmode = none;
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = false;
		// SrcBlend = one;
		// DestBlend = one;
		PixelShader = compile ps_2_0 dropPS();	
    }
    // now lay down ambient, reflection, and Z
    //    subsequent passes need not save Z, and they will
    //	be faster because of implicit Z occlusion
    pass ambiRefl 
    {		
		VertexShader = compile vs_2_0 ambiReflVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 ambiReflPS();
    }
    // directional light contribution
    pass distantlight1 
    {		
		VertexShader = compile vs_2_0 distantlightVS(LightDirD);
		ZEnable = true;
		ZWriteEnable = false;
		ZFunc = LessEqual;
		CullMode = None;
		AlphaBlendEnable = true;
		SrcBlend = One;
		DestBlend = One;
		PixelShader = compile ps_2_0 lampPS(LightColorD,LightIntensityD);
    }
}

//////// NO dropshadow

technique just3d
{
    // first lay down ambient, reflection, and Z
    //    subsequent passes need not save Z, and they will
    //	be faster because of implicit Z occlusion
    pass ambiRefl 
    {		
		VertexShader = compile vs_2_0 ambiReflVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 ambiReflPS();
    }
    // directional light contribution
    pass distantlight1 
    {		
		VertexShader = compile vs_2_0 distantlightVS(LightDirD);
		ZEnable = true;
		ZWriteEnable = false;
		ZFunc = LessEqual;
		CullMode = None;
		AlphaBlendEnable = true;
		SrcBlend = One;
		DestBlend = One;
		PixelShader = compile ps_2_0 lampPS(LightColorD,LightIntensityD);
    }
}

/***************************** eof ***/
