/*********************************************************************NVMH3****
File:  $Id: scene_EasyBake.fx,v 1.1 2004/03/22 17:48:02 holger Exp $

Copyright NVIDIA Corporation 2002
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
    Simple pixel-shaded point-lit plastic without falloff -- but with
    	lighting baked into a texture. See "bakeliteTimes9" for example
	of application to instantiations
    EffectEdit okay.

<scene_commands>
<clear color="0.0,0.0,0.0,0" depth="1.0"/>
<setcolortarget texture="BakeTex"/>
<setdepthstenciltarget texture="DepthBuffer"/>
	<clear color="0.0,0.0,0.0,0" depth="1.0"/>
	<settechnique name="bakelite"/>
	<setpass name="bake"/>
	<draw type="objects_override"/>
<setcolortarget/>
<setdepthstenciltarget/>
	<setpass name="use"/>
	<draw type="objects_override"/>
</scene_commands>

******************************************************************************/

string XFile = "tiger.x";

/************* UN-TWEAKABLES **************/

half4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; > = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
half4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
half4x4 WorldXf : World < string UIWidget="None"; >;
half4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

/*********** Tweakables **********************/

half3 LightPos : Position
<
    string Object = "PointLight";
    string Space = "World";
> = {-1.0f, 1.0f, -0.4f};

half3 LightColor
<
    string UIName =  "Light Color";
    string UIWidget = "Color";
    string Object = "PointLight";
> = {1.0f, 1.0f, 1.0f};

//////

half3 SurfColor : DIFFUSE
<
    string UIName =  "Surface Color";
    string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f};

half3 AmbiColor : Ambient
<
    string UIName =  "Ambi Color";
    string UIWidget = "Color";
> = {0.1f, 0.1f, 0.1f};

half Ks
<
    string UIWidget = "slider";
    half UIMin = 0.0;
    half UIMax = 1.0;
    half UIStep = 0.01;
    string UIName =  "specular intensity";
> = 0.5;

half SpecExpon : SpecularPower
<
    string UIWidget = "slider";
    half UIMin = 1.0;
    half UIMax = 128.0;
    half UIStep = 1.0;
    string UIName =  "specular power";
> = 30.0;

// for instances
half Spread
<
    string UIWidget = "slider";
    half UIMin = 0.1;
    half UIMax = 15.0;
    half UIStep = 0.1;
    string UIName =  "Instance Spacing";
> = 2.5;

/// texture ///////////////////////////////////

texture colorTexture : DIFFUSE
<
    string ResourceName = "tiger.bmp";
    string TextureType = "2D";
>;

sampler2D ColorSampler = sampler_state
{
    Texture = <colorTexture>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
};

///////////

#define BAKE_SIZE 256

texture BakeTex : RENDERCOLORTARGET < 
    int width = BAKE_SIZE;
    int height = BAKE_SIZE;
    int MIPLEVELS = 1;
    string format = "X8R8G8B8";
    string UIWidget = "None";
>;

texture DepthBuffer : RENDERDEPTHSTENCILTARGET <
	int width = BAKE_SIZE;
	int height = BAKE_SIZE;
	string format = "D24S8";
>;

sampler BakeSampler = sampler_state 
{
    texture = <BakeTex>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
    half3 Position    : POSITION;
    half4 UV        : TEXCOORD0;
    half4 Normal    : NORMAL;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    half4 HPosition    : POSITION;
    half2 UV        : TEXCOORD0;
    half3 LightVec    : TEXCOORD1;
    half3 WorldNormal    : TEXCOORD2;
    half3 WorldEyeVec    : TEXCOORD3;
};

/* data passed from vertex shader to pixel shader */
struct bakedVertexOutput {
    half4 HPosition    : POSITION;
    half2 UV        : TEXCOORD0;
};

/*********** vertex shader ******/

// use this if we are just rendering normally
vertexOutput mainVS(appdata IN) {
    vertexOutput OUT;
    OUT.WorldNormal = mul(IN.Normal, WorldITXf).xyz;
    half4 Po = half4(IN.Position.xyz,1.0);
    half3 Pw = mul(Po, WorldXf).xyz;
    OUT.LightVec = (LightPos - Pw);
    OUT.WorldEyeVec = (ViewIXf[3].xyz - Pw);
    OUT.HPosition = mul(Po, WvpXf);
    OUT.UV = IN.UV.xy;
    return OUT;
}

// use this to create baked lighting textures
vertexOutput bakeVS(appdata IN) {
    vertexOutput OUT;
    OUT.WorldNormal = mul(IN.Normal, WorldITXf).xyz;
    half4 Po = half4(IN.Position.xyz,1.0);
    half3 Pw = mul(Po, WorldXf).xyz;
    OUT.LightVec = (LightPos - Pw);
    OUT.WorldEyeVec = (ViewIXf[3].xyz - Pw);
    // OUT.HPosition = mul(Po, WvpXf);
    half2 nuPos = half2(IN.UV.x,1-IN.UV.y);
    nuPos = 2.0*(nuPos-0.5);
    OUT.HPosition = half4(nuPos,1.0,1.0);
    OUT.UV = IN.UV.xy;
    return OUT;
}

// use this to apply pre-baked lighting textures
bakedVertexOutput minimalVS(appdata IN,uniform half3 offset) {
    bakedVertexOutput OUT;
    half4 Po = half4((IN.Position.xyz+(Spread*offset)),1.0);
    OUT.HPosition = mul(Po, WvpXf);
    OUT.UV = IN.UV.xy;
    return OUT;
}

/********* pixel shader ********/

// simple plastic shading (DiffColor could be textured or not)
half3 sharedPS(vertexOutput IN,half3 DiffColor) :COLOR {
    half3 Ln = normalize(IN.LightVec);
    half3 Nn = normalize(IN.WorldNormal);
    half3 Vn = normalize(IN.WorldEyeVec);
    half3 Hn = normalize(Vn + Ln);
    half4 lv = lit(dot(Ln,Nn),dot(Hn,Nn),SpecExpon);
    half3 diffContrib = DiffColor * (lv.y*LightColor + AmbiColor);
    half3 specContrib = Ks * lv.z * LightColor;
    return diffContrib + specContrib;
}

// simple plastic shading with flat color
half4 untexturedPS(vertexOutput IN) :COLOR {
    return half4(sharedPS(IN,SurfColor),1);
}

// simple plastic shading with texture
half4 texturedPS(vertexOutput IN) :COLOR {
    return half4(sharedPS(IN,(SurfColor*tex2D(ColorSampler,IN.UV).xyz)),1);
}

// just use pre-baked texture for all color/lighting
half4 prebakedPS(bakedVertexOutput IN) :COLOR {
    return half4(tex2D(BakeSampler,IN.UV).xyz,1);
}

/*************/

// bake lighting to texture, then apply it
technique bakelite
{
    pass bake
    {        
        VertexShader = compile vs_2_0 bakeVS();
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_2_0 texturedPS();
    }
    pass use 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(0,0,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
}

// bake lighting to texture, then apply it
technique bakeliteTimes9
{
    pass bake 
    {        
        VertexShader = compile vs_2_0 bakeVS();
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_2_0 texturedPS();
    }
    pass use00 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(-1,-1,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use01 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(-1,0,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use02 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(-1,1,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use10 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(0,-1,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use11 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(0,0,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use12 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(0,1,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use20 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(1,-1,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use21 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(1,0,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
    pass use22 
    {        
        VertexShader = compile vs_1_1 minimalVS(half3(1,1,0));
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_1_1 prebakedPS();
    }
}

// just show baking results
technique bake_tex_only
{
    pass p0 
    {        
        VertexShader = compile vs_2_0 bakeVS();
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_2_0 texturedPS();
    }
}

// "straight" lighting
technique plastic_textured
{
    pass p0 
    {        
        VertexShader = compile vs_2_0 mainVS();
        ZEnable = true;
        ZWriteEnable = true;
        CullMode = None;
        PixelShader = compile ps_2_0 texturedPS();
    }
}

/***************************** eof ***/
