/*********************************************************************NVMH3****
File:  $Id: scene_Mandelbrot.fx,v 1.1 2004/03/22 17:48:02 holger Exp $

Copyright NVIDIA Corporation 2002
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
    Mandelbrot set browser.
	Scene geometry is ignored.
	For more speed, reduce MAXIMUM_ITERATIONS
	For more detail, increase MAXIMUM_ITERATIONS
    Not made for EffectEdit.

<scene_commands>
<setcolortarget/>
<clear color="0,0,0"/>
<settechnique name="mandy"/>
<draw type="buffer"/>
</scene_commands>

******************************************************************************/

///////////////////////////////////////////////////////////
/////////////////////////////////////// Tweakables ////////
///////////////////////////////////////////////////////////

float Scale <
	string UIWidget = "slider";
	float UIMin = 0.01;
	float UIMax = 4.0;
	float UIStep = 0.01;
> = 0.11f;

// float2 Center = {0.5,0.1};
float CenterX <
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 1.0;
	float UIStep = 0.001;
> = 0.709f;

float CenterY <
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 1.0;
	float UIStep = 0.001;
> = 0.350f;

float3 InColor <
	string UIWidget = "Color";
> = {0,0,0};

float3 OutColorA <
	string UIWidget = "Color";
> = {1,0,.3};

float3 OutColorB <
	string UIWidget = "Color";
> = {.2,1,0};

#define MIXIMUM_ITERATIONS 20

///////////////////////////////////////////////////////////
/////////////////////////////////// data structures ///////
///////////////////////////////////////////////////////////

struct VS_OUTPUT {
    float4 Position : POSITION;
    float2 TexCoord0	: TEXCOORD0;
};

///////////////////////////////////////////////////////////
////////////////////////////////// vertex shader //////////
///////////////////////////////////////////////////////////

VS_OUTPUT VS_Quad(float3 Position : POSITION, 
			float3 TexCoord : TEXCOORD0)
{
    VS_OUTPUT OUT = (VS_OUTPUT)0;
    OUT.Position = float4(Position, 1);
    OUT.TexCoord0 = TexCoord.xy;
    return OUT;
}

/////////////////////////////////////////////////////
////////////////////////////////// pixel shader /////
/////////////////////////////////////////////////////

float4 mandyPS(VS_OUTPUT IN) : COLOR {
    //float3 c = mandelbrot_color(IN.TexCoord0);
    float2 pos = frac(IN.TexCoord0);
    float real = ((pos.x - 0.5)*Scale)-CenterX;
    float imag = ((pos.y - 0.5)*Scale)-CenterY;
    float Creal = real;
    float Cimag = imag;
    float r2 = 0;
    float i;
    for (i=0; (i<MIXIMUM_ITERATIONS) && (r2<4.0); i++) {
		float tempreal = real;
		real = (tempreal*tempreal) - (imag*imag) + Creal;
		imag = 2*tempreal*imag + Cimag;
		r2 = (real*real) + (imag*imag);
    }
    float3 finalColor;
    if (r2 < 4) {
       finalColor = InColor;
    } else {
    	finalColor = lerp(OutColorA,OutColorB,frac(i * 0.05));
    }
    return float4(finalColor,1);
}

///////////////////////////////////////////////////////////
/////////////////////////////////////// technique /////////
///////////////////////////////////////////////////////////

technique mandy {
    pass pman {
	VertexShader = compile vs_1_1 VS_Quad();
	cullmode = none;
	ZEnable = false;
	ZWriteEnable = false;
	AlphaBlendEnable = true;
	SrcBlend = one;
	DestBlend = one;
	PixelShader = compile ps_2_a mandyPS();
    }
}

//////////////////////////////////////////////////////////
////////////////////////////////////////////////// eof ///
//////////////////////////////////////////////////////////
