/*********************************************************************NVMH3****
File:  $Id: scene_mouseDot.fx,v 1.1 2004/03/22 17:48:02 holger Exp $

Copyright NVIDIA Corporation 2002
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
	Fun with mice.
	Click on the screen, move the mouse around.
	Scene geometry is ignored.

<scene_commands>
<clear color="0,0,0" depth="1.0"/>
<settechnique name="mouseBloom"/>
<draw type="buffer"/>
</scene_commands>
******************************************************************************/

float4 MouseL : LEFTMOUSEDOWN < string UIWidget="None"; >;
// float4 MouseR : RIGHTMOUSEDOWN < string UIWidget="None"; >;
float3 MousePos : MOUSEPOSITION < string UIWidget="None"; >;
float Time : TIME < string UIWidget = "None"; >;

float TimeScale <
	string UIWidget = "slider";
	float UIMin = 0.01;
	float UIMax = 4.0;
	float UIStep = 0.01;
> = 0.6f;

/****************************************/
/*** Connector Struct *******************/
/****************************************/

struct VS_OUTPUT {
    float4 Position : POSITION;
    float2 TexCoord0	: TEXCOORD0;
};

/****************************************/
/*** Shaders ****************************/
/****************************************/

VS_OUTPUT VS_Quad(float3 Position : POSITION, 
			float3 TexCoord : TEXCOORD0)
{
    VS_OUTPUT OUT = (VS_OUTPUT)0;
    OUT.Position = float4(Position, 1);
    OUT.TexCoord0 = float4(TexCoord, 1); 
    return OUT;
}

float4 PS(VS_OUTPUT IN) : COLOR
{
    float2 delta = IN.TexCoord0.xy-MouseL.xy;
    float dd = length(delta);
    float dt = (Time - MouseL.w)*TimeScale;
    float dr = min(1,max(0,1-abs(dd-dt)));
    // dr = dr * MouseL.z; // if this line is commented, effect only happens when mouse is pressed
    float2 mDelta = 0.5+(MousePos.xy - MouseL.xy);
    return float4(dr*mDelta,dr,1);
}

/****************************************/
/*** Technique **************************/
/****************************************/

technique mouseBloom {
	pass p0 {
		VertexShader = compile vs_1_1 VS_Quad();
		ZEnable = false;
		ZWriteEnable = false;
		CullMode = None;
		PixelShader  = compile ps_2_a PS();
	}
}

/****************************************/
/******************************* eof ****/
/****************************************/
