/*********************************************************************NVMH3****
File:  $Id: scene_paint3D.fx,v 1.1 2004/03/22 17:48:02 holger Exp $

Copyright NVIDIA Corporation 2002
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
	An .FX Paint Program that needs scene geometry to use "showUV.fx"
	Draw with the left mouse button.
	To clear screen, just make the brush big and paint everything.
	Resizing the window will mess up your drawing.
	Brush strokes will change in size and opacity over time, set "FadeTime"
		to a high value for more even (though less expressive) strokes.
	For smoothest performance, close the Materials and Textures Panels,
		and close the "Parameters" dropdown in the Properties window except
		when changing brush size. This will keep FX Composer drawing instead
		of updating those windows.

<scene_commands>
<setcolortarget texture="UVMap"/>
<setdepthstenciltarget texture="DepthTarget"/>
<clear color="0.0,0.0,0.0,0.0" depth="1.0"/>
<draw type="objects"/>
<setcolortarget texture="BufMap"/>
<clear color="0.0,0.0,0.1,0.0" depth="1.0"/>
<settechnique name="restorePaint"/>
<draw type="buffer"/>
<setcolortarget texture="PaintMap"/>
<settechnique name="brushStroke"/>
<draw type="buffer"/>
<setcolortarget/>
<clear color="0.0,0.0,0.1,0.0" depth="1.0"/>
<settechnique name="showTex"/>
<draw type="buffer"/>
</scene_commands>

******************************************************************************/

// #define USE_TIMER

half4 MouseL : LEFTMOUSEDOWN < string UIWidget="None"; >;
half3 MousePos : MOUSEPOSITION < string UIWidget="None"; >;
#ifdef USE_TIMER
half Timer : TIME < string UIWidget = "None"; >;
#endif /* USE_TIMER */

half Opacity <
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 1.0;
	half UIStep = 0.01;
> = 0.45f;

#ifdef USE_TIMER
half BrushSizeStart <
	string UIWidget = "slider";
	half UIMin = 0.001;
	half UIMax = 0.15;
	half UIStep = 0.001;
> = 0.07f;

half BrushSizeEnd <
	string UIWidget = "slider";
	half UIMin = 0.001;
	half UIMax = 0.15;
	half UIStep = 0.001;
> = 0.01f;

half FadeTime <
	string UIWidget = "slider";
	half UIMin = 0.1;
	half UIMax = 10.0;
	half UIStep = 0.1;
> = 2.00f;

#else /* !USE_TIMER */
half BrushSize <
	string UIWidget = "slider";
	half UIMin = 0.001;
	half UIMax = 0.15;
	half UIStep = 0.001;
> = 0.07f;
#endif /* !USE_TIMER */

half3 PaintColor <
	string UIWidget = "Color";
> = {0.4f, 0.3f, 1.0f};

///////////////////////////////////////////////////////////
///////////////////////////// Render-to-Texture Data //////
///////////////////////////////////////////////////////////

texture UVMap : RENDERCOLORTARGET < 
    float2 ViewPortDimensions = {1.0,1.0};
    int MIPLEVELS = 1;
    string format = "A8R8G8B8";
    string UIWidget = "None";
>;

texture DepthTarget : RENDERDEPTHSTENCILTARGET < 
    float2 ViewPortDimensions = {1.0,1.0};
    string format = "D24S8";
    string UIWidget = "None";
>;

sampler UVSampler = sampler_state {
    texture = <UVMap>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

texture PaintMap : RENDERCOLORTARGET < 
    float2 ViewPortDimensions = {1.0,1.0};
    int MIPLEVELS = 1;
    string format = "X8R8G8B8";
    string UIWidget = "None";
>;

sampler PaintSampler = sampler_state {
    texture = <PaintMap>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

texture BufMap : RENDERCOLORTARGET < 
    float2 ViewPortDimensions = {1.0,1.0};
    int MIPLEVELS = 1;
    string format = "X8R8G8B8";
    string UIWidget = "None";
>;

sampler BufSampler = sampler_state {
    texture = <BufMap>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

/****************************************/
/*** The dead-simple shaders: ***********/
/****************************************/

struct VS_OUTPUT {
    half4 Position : POSITION;
    half2 TexCoord0	: TEXCOORD0;
};

VS_OUTPUT VS_Quad(half3 Position : POSITION, 
			half3 TexCoord : TEXCOORD0)
{
    VS_OUTPUT OUT = (VS_OUTPUT)0;
    OUT.Position = half4(Position, 1);
    OUT.TexCoord0 = half4(TexCoord, 1); 
    return OUT;
}

half4 restorePaintPS(VS_OUTPUT IN) : COLOR
{
	half4 c = tex2D(PaintSampler,IN.TexCoord0);
    return c;
}

half4 brushPS(VS_OUTPUT IN) : COLOR
{
	half4 uvmap = tex2D(UVSampler,MousePos.xy);
    half2 delta = IN.TexCoord0.xy-uvmap.xy;
	half dd = uvmap.w * Opacity;
#ifdef USE_TIMER
	half fadeout = 1 - min(1,max(0,(Timer - MouseL.w))/FadeTime);
	half ds = lerp(BrushSizeEnd,BrushSizeStart,fadeout);
    dd *= fadeout;
    dd *= MouseL.z*(1-min(length(delta)/ds,1));
#else /* USE_TIMER */
    dd *= MouseL.z*(1-min(length(delta)/BrushSize,1));
#endif /* USE_TIMER */
    return dd*half4(PaintColor,1);
}

half4 remapPS(VS_OUTPUT IN) : COLOR
{
	half4 uvmap = tex2D(UVSampler,IN.TexCoord0);
	half3 c = uvmap.w * tex2D(PaintSampler,uvmap.xy).xyz;
    return half4(c,uvmap.w);
}

////////////////// Technique ////////

// fill fb with previous state of paint texture
technique restorePaint {
	pass p0
	{
		VertexShader = compile vs_1_1 VS_Quad();
		PixelShader  = compile ps_2_a restorePaintPS();
		AlphaBlendEnable = false;
		ZEnable = false;
	}
}

// paint into paint texture
technique brushStroke {
	pass p0  {
		VertexShader = compile vs_1_1 VS_Quad();
		PixelShader  = compile ps_2_a brushPS();
		AlphaBlendEnable = true;
		SrcBlend = One;
		DestBlend = InvSrcAlpha;
		ZEnable = false;
	}
}

// draw revised texture onto surface
technique showTex {
	pass p0 {
		VertexShader = compile vs_1_1 VS_Quad();
		PixelShader  = compile ps_2_a remapPS();
		AlphaBlendEnable = false;
		ZEnable = false;
	}
}

/////////////////////////////////////////////
///////////////////////////////////// eof ///
/////////////////////////////////////////////
